/*
 * $Id$
 *
 * Copyright (C) 2010-2011 Tomasz Bolesław CEDRO 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Written by Tomasz Bolesław CEDRO, 2011
 *
 */

#include <sysdep.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <urjtag/error.h>
#include <urjtag/chain.h>
#include <urjtag/cmd.h>
#include <libswd/libswd.h>

#include "cmd.h"

static int
cmd_swd_run (urj_chain_t *chain, char *params[])
{
    int i, j, res=0, payload32=0, *idcode;
    char APnDP=0, RnW=0, addr=0, parity=0, payload8=0;

    if ((i = urj_cmd_params (params)) < 3)
    {
        urj_error_set (URJ_ERROR_SYNTAX,
                       "%s: #parameters should be >= %d , not %d",
                       params[0], 3, urj_cmd_params (params));
        return URJ_STATUS_FAIL;
    }

//    if (urj_cmd_test_cable (chain) != URJ_STATUS_OK)
//        return URJ_STATUS_FAIL;
    
    for (j = 1; j < i; j++)
    {
        urj_log(URJ_LOG_LEVEL_DEBUG, "Debug in cmd_swd(): ");
        if (strncasecmp(params[j], "read", 1)==0){
            urj_log(URJ_LOG_LEVEL_DEBUG, "read ");
            /* Read operation require one parameter. */
            if (j+1<i){
                if (strncasecmp(params[j+1], "ack", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "ack ");
                    char *ack;
                    res=swd_bus_read_ack(chain->swdctx,SWD_OPERATION_EXECUTE, &ack);
                    if (res<0) break;
                    printf("0x%x ", *ack);
                    j+=1;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                } else if (strncasecmp(params[j+1], "data", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "data ");
                    char *parity;
                    int *data;
                    res=swd_bus_read_data_p(chain->swdctx, SWD_OPERATION_EXECUTE, &data, &parity);
                    if (res<0) break;
                    printf("0x%x 0x%x ", *data, *parity);
                    j+=1;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                }
            }
        } else if (strncasecmp(params[j], "write", 1)==0){
            urj_log(URJ_LOG_LEVEL_DEBUG, "write ");
            /* Write operation that requires two parameters. */
            if (j+2<i){
               if (strncasecmp(params[j+1], "control", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "control ");
                    payload8=(char)atoi(params[j+2]);
                    res=swd_bus_write_control(chain->swdctx, SWD_OPERATION_EXECUTE, &payload8, 8);
                    if (res<0) break;
                    printf("0x%x ", payload8);
                    j+=2;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                } else if (strncasecmp(params[j+1], "data", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "data ");
                    payload32=atoi(params[j+2]);
                    res=swd_bus_write_data_ap(chain->swdctx, SWD_OPERATION_EXECUTE, &payload32);
                    if (res<0) break;
                    printf("0x%x ", payload32);
                    j+=2;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                } 
            }
            /* Write operation that require three parameters. */
            if (j+3<i){
                if (strncasecmp(params[j+1], "dataparity", 10)==0 || strncasecmp(params[j+1], "dp", 2)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "dataparity ");
                    payload32=atoi(params[j+2]);
                    parity=(char)atoi(params[j+3]);
                    res=swd_bus_write_data_p(chain->swdctx, SWD_OPERATION_EXECUTE, &payload32, &parity); 
                    printf("0x%x 0x%x ", payload32, parity);
                    j+=3;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                }
            }               
            /* Write operation that require four parameters. */
            if (j+4<i){
                if (strncasecmp(params[j+1], "request", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "request ");
                    APnDP=atoi(params[j+2]);
                    RnW=atoi(params[j+3]);
                    addr=atoi(params[j+4]);
                    res=swd_bus_write_request(chain->swdctx, SWD_OPERATION_EXECUTE, &APnDP, &RnW, &addr);
                    if (res<0) break;
                    printf("0x%x ", payload32); 
                    j+=4;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                }
            }
        } else if (strncasecmp(params[j], "dap", 1)==0){
            urj_log(URJ_LOG_LEVEL_DEBUG, "dap ");
            /* DAP operations that require one parameter. */
            if (j+1<i){
                if (strncasecmp(params[j+1], "select", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "select ");
                    res=swd_dap_select(chain->swdctx, SWD_OPERATION_EXECUTE);
                    if (res<0) break;
                    j+=1;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                } else if (strncasecmp(params[j+1], "reset", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "reset ");
                    res=swd_dap_reset(chain->swdctx, SWD_OPERATION_EXECUTE);
                    if (res<0) break;
                    j+=1;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                } else if (strncasecmp(params[j+1], "detect", 1)==0){
                    urj_log(URJ_LOG_LEVEL_DEBUG, "detect ");
                    res=swd_dap_detect(chain->swdctx, SWD_OPERATION_EXECUTE, &idcode);
                    if (res<0) break;
                    printf("0x%x ", *idcode);
                    j+=1;
                    urj_log(URJ_LOG_LEVEL_DEBUG, "OK\n");
                    continue;
                }
            }
            urj_warning("cmd_swd_warning: unknown parameter!\n");
        }

        if (res<0){
            urj_log(URJ_LOG_LEVEL_DEBUG, "returns %d\n", res);        
            urj_error_set(URJ_ERROR_TRANSPORT, "cmd_swd(): transport error %s\n", swd_error_string(res));
            return URJ_STATUS_FAIL;        
        }
        if (res==URJ_STATUS_FAIL) {
            urj_error_set (URJ_ERROR_SYNTAX,
                           "Syntax error! Type \"help swd\" for help.\n");
            return URJ_STATUS_FAIL;
        }
    }

    return URJ_STATUS_OK;
}

static void
cmd_swd_help (void)
{
    urj_log (URJ_LOG_LEVEL_NORMAL,
             _("Usage: %s DIRECTION COMMAND <parameters>\n"
               "Operates on the SWD transport layer using libswd.\n"
               "\n"
               " operations:\n"
               "  read  {ack, data}\n"
               "  write {control, data} payload\n"
               "  write request APnDP RnW addr\n"
               "  write {dataparity} payload parity\n"
               "  dap   {reset, detect}\n"
               "\n"),
             "swd");
}

const urj_cmd_t urj_cmd_swd = {
    "swd",
    N_("Serial Wire Debug transport layer operations"),
    cmd_swd_help,
    cmd_swd_run
};
